@@ -36,13 +36,12 @@
 #include <libtracker-fts/tracker-fts.h>
 #endif
 
-#include <libtracker-db/tracker-db-interface-sqlite.h>
-#include <libtracker-db/tracker-db-manager.h>
-#include <libtracker-db/tracker-db-journal.h>
-
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
+#include "tracker-db-interface-sqlite.h"
+#include "tracker-db-manager.h"
+#include "tracker-db-journal.h"
 #include "tracker-namespace.h"
 #include "tracker-ontologies.h"
 #include "tracker-ontology.h"
@@ -112,12 +111,18 @@ set_secondary_index_for_single_value_property (TrackerDBInterface *iface,
                                                const gchar        *second_field_name,
                                                gboolean            enabled)
 {
+	g_debug ("Dropping index:  DROP INDEX IF EXISTS \"%s_%s\"",
+	         service_name, field_name);
+
 	tracker_db_interface_execute_query (iface, NULL,
 	                                    "DROP INDEX IF EXISTS \"%s_%s\"",
 	                                    service_name,
 	                                    field_name);
 
 	if (enabled) {
+		g_debug ("Creating index: CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\", \"%s\")",
+		         service_name, field_name, service_name, field_name, second_field_name);
+
 		tracker_db_interface_execute_query (iface, NULL,
 		                                    "CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\", \"%s\")",
 		                                    service_name,
@@ -134,12 +139,18 @@ set_index_for_single_value_property (TrackerDBInterface *iface,
                                      const gchar        *field_name,
                                      gboolean            enabled)
 {
+	g_debug ("Dropping index: DROP INDEX IF EXISTS \"%s_%s\"",
+	         service_name, field_name);
+
 	tracker_db_interface_execute_query (iface, NULL,
 	                                    "DROP INDEX IF EXISTS \"%s_%s\"",
 	                                    service_name,
 	                                    field_name);
 
 	if (enabled) {
+		g_debug ("Creating index: CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\")",
+		         service_name, field_name, service_name, field_name);
+
 		tracker_db_interface_execute_query (iface, NULL,
 		                                    "CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\")",
 		                                    service_name,
@@ -380,6 +391,7 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 					g_critical ("%s: Duplicate definition of class %s", ontology_path, subject);
 				} else {
 					/* Reset for a correct post-check */
+					tracker_class_reset_domain_indexes (class);
 					tracker_class_set_notify (class, FALSE);
 				}
 				return;
@@ -416,6 +428,7 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 					g_critical ("%s: Duplicate definition of property %s", ontology_path, subject);
 				} else {
 					/* Reset for a correct post-check */
+					tracker_property_reset_domain_indexes (property);
 					tracker_property_set_indexed (property, FALSE);
 					tracker_property_set_secondary_index (property, NULL);
 					tracker_property_set_writeback (property, FALSE);
@@ -516,6 +529,87 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 		}
 
 		tracker_class_set_notify (class, (strcmp (object, "true") == 0));
+	} else if (g_strcmp0 (predicate, TRACKER_PREFIX "domainIndex") == 0) {
+		TrackerClass *class;
+		TrackerProperty *property;
+		TrackerProperty **properties;
+		gboolean ignore = FALSE;
+		gboolean had = FALSE;
+		guint n_props, i;
+
+		class = tracker_ontologies_get_class_by_uri (subject);
+
+		if (class == NULL) {
+			g_critical ("%s: Unknown class %s", ontology_path, subject);
+			return;
+		}
+
+		property = tracker_ontologies_get_property_by_uri (object);
+
+		if (property == NULL) {
+
+			/* In this case the import of the TTL will still make the introspection
+			 * have the URI set as a tracker:domainIndex for class. The critical
+			 * will have happened, but future operations might not cope with this
+			 * situation. TODO: add error handling so that the entire ontology
+			 * change operation is discarded, for example ignore the entire
+			 * .ontology file and rollback all changes that happened. I would
+			 * prefer a hard abort() here over a g_critical(), to be honest.
+			 *
+			 * Of course don't we yet allow just anybody to alter the ontology
+			 * files. So very serious is this lack of thorough error handling
+			 * not. Let's just not make mistakes when changing the .ontology
+			 * files for now. */
+
+			g_critical ("%s: Unknown property %s for tracker:domainIndex in %s."
+			            "Don't release this .ontology change!",
+			            ontology_path, object, subject);
+			return;
+		}
+
+		if (tracker_property_get_multiple_values (property)) {
+			g_critical ("%s: Property %s has multiple values while trying to add it as tracker:domainIndex in %s, this isn't supported",
+			            ontology_path, object, subject);
+			return;
+		}
+
+		properties = tracker_ontologies_get_properties (&n_props);
+		for (i = 0; i < n_props; i++) {
+			if (tracker_property_get_domain (properties[i]) == class &&
+			    properties[i] == property) {
+				g_critical ("%s: Property %s is already a first-class property of %s while trying to add it as tracker:domainIndex",
+				            ontology_path, object, subject);
+			}
+		}
+
+		properties = tracker_class_get_domain_indexes (class);
+		while (*properties) {
+			if (property == *properties) {
+				g_debug ("%s: Property %s already a tracker:domainIndex in %s",
+				         ontology_path, object, subject);
+				ignore = TRUE;
+			}
+			properties++;
+		}
+
+		properties = tracker_class_get_last_domain_indexes (class);
+		if (properties) {
+			while (*properties) {
+				if (property == *properties) {
+					had = TRUE;
+				}
+				properties++;
+			}
+		}
+
+		if (!ignore) {
+			if (!had) {
+				tracker_property_set_is_new_domain_index (property, in_update);
+			}
+			tracker_class_add_domain_index (class, property);
+			tracker_property_add_domain_index (property, class);
+		}
+
 	} else if (g_strcmp0 (predicate, TRACKER_PREFIX "writeback") == 0) {
 		TrackerProperty *property;
 
@@ -736,9 +830,120 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 	}
 }
 
+
+static void
+check_for_deleted_domain_index (TrackerClass *class)
+{
+	TrackerProperty **last_domain_indexes;
+	GSList *hfound = NULL, *deleted = NULL;
+
+	last_domain_indexes = tracker_class_get_last_domain_indexes (class);
+
+	if (!last_domain_indexes) {
+		return;
+	}
+
+	while (*last_domain_indexes) {
+		TrackerProperty *last_domain_index = *last_domain_indexes;
+		gboolean found = FALSE;
+		TrackerProperty **domain_indexes;
+
+		domain_indexes = tracker_class_get_domain_indexes (class);
+
+		while (*domain_indexes) {
+			TrackerProperty *domain_index = *domain_indexes;
+			if (last_domain_index == domain_index) {
+				found = TRUE;
+				hfound = g_slist_prepend (hfound, domain_index);
+				break;
+			}
+			domain_indexes++;
+		}
+
+		if (!found) {
+			deleted = g_slist_prepend (deleted, last_domain_index);
+		}
+
+		last_domain_indexes++;
+	}
+
+
+	if (deleted) {
+		GSList *l;
+		TrackerProperty **properties;
+		guint n_props, i;
+
+		tracker_class_set_db_schema_changed (class, TRUE);
+
+		properties = tracker_ontologies_get_properties (&n_props);
+		for (i = 0; i < n_props; i++) {
+			if (tracker_property_get_domain (properties[i]) == class &&
+			    !tracker_property_get_multiple_values (properties[i])) {
+
+				/* These aren't domain-indexes, but it's just a flag for the
+				 * functionality that'll recreate the table to know that the
+				 * property must be involved in the recreation and copy */
+
+				tracker_property_set_is_new_domain_index (properties[i], TRUE);
+			}
+		}
+
+		for (l = hfound; l != NULL; l = l->next) {
+			TrackerProperty *prop = l->data;
+			g_debug ("Ontology change: keeping tracker:domainIndex: %s",
+			         tracker_property_get_name (prop));
+			tracker_property_set_is_new_domain_index (prop, TRUE);
+		}
+
+		for (l = deleted; l != NULL; l = l->next) {
+			GError *error = NULL;
+			TrackerProperty *prop = l->data;
+
+			g_debug ("Ontology change: deleting tracker:domainIndex: %s",
+			         tracker_property_get_name (prop));
+			tracker_property_del_domain_index (prop, class);
+			tracker_class_del_domain_index (class, prop);
+
+			tracker_data_delete_statement (NULL, tracker_class_get_uri (class),
+			                               TRACKER_PREFIX "domainIndex",
+			                               tracker_property_get_uri (prop),
+			                               &error);
+
+			if (error) {
+				g_critical ("Ontology change, %s", error->message);
+				g_clear_error (&error);
+			} else {
+				tracker_data_update_buffer_flush (&error);
+				if (error) {
+					g_critical ("Ontology change, %s", error->message);
+					g_clear_error (&error);
+				}
+			}
+		}
+
+		g_slist_free (deleted);
+	}
+
+	g_slist_free (hfound);
+}
+
 void
-tracker_data_ontology_process_changes (GPtrArray *seen_classes,
-                                       GPtrArray *seen_properties)
+tracker_data_ontology_process_changes_pre_db (GPtrArray *seen_classes,
+                                               GPtrArray *seen_properties)
+{
+	gint i;
+	if (seen_classes) {
+		for (i = 0; i < seen_classes->len; i++) {
+			TrackerClass *class = g_ptr_array_index (seen_classes, i);
+			check_for_deleted_domain_index (class);
+
+		}
+	}
+}
+
+void
+tracker_data_ontology_process_changes_post_db (GPtrArray *seen_classes,
+                                               GPtrArray *seen_properties)
 {
 	gint i;
 
@@ -858,6 +1063,13 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 	}
 }
 
+void
+tracker_data_ontology_process_changes_post_import (GPtrArray *seen_classes,
+                                                   GPtrArray *seen_properties)
+{
+	return;
+}
+
 void
 tracker_data_ontology_free_seen (GPtrArray *seen)
 {
@@ -992,7 +1204,8 @@ load_ontology_from_journal (GHashTable **classes_out,
 	properties = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 	                                    NULL, (GDestroyNotify) g_object_unref);
 
-	id_uri_map = g_hash_table_new (g_direct_hash, g_direct_equal);
+	id_uri_map = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+	                                    NULL, g_free);
 
 	while (tracker_db_journal_reader_next (NULL)) {
 		TrackerDBJournalEntryType type;
@@ -1003,7 +1216,7 @@ load_ontology_from_journal (GHashTable **classes_out,
 			const gchar *uri;
 
 			tracker_db_journal_reader_get_resource (&id, &uri);
-			g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), (gpointer) uri);
+			g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), g_strdup (uri));
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
 			/* end of initial transaction => end of ontology */
 			break;
@@ -1213,7 +1426,7 @@ class_add_super_classes_from_db (TrackerDBInterface *iface,
 	g_object_unref (stmt);
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
 			TrackerClass *super_class;
 			const gchar *super_class_uri;
 
@@ -1226,6 +1439,45 @@ class_add_super_classes_from_db (TrackerDBInterface *iface,
 	}
 }
 
+
+static void
+class_add_domain_indexes_from_db (TrackerDBInterface *iface,
+                                  TrackerClass       *class)
+{
+	TrackerDBStatement *stmt;
+	TrackerDBCursor *cursor;
+	GError *error = NULL;
+
+	stmt = tracker_db_interface_create_statement (iface, &error,
+	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"tracker:domainIndex\") "
+	                                              "FROM \"rdfs:Class_tracker:domainIndex\" "
+	                                              "WHERE ID = (SELECT ID FROM Resource WHERE Uri = ?)");
+
+	if (!stmt) {
+		g_critical ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	tracker_db_statement_bind_text (stmt, 0, tracker_class_get_uri (class));
+	cursor = tracker_db_statement_start_cursor (stmt, NULL);
+	g_object_unref (stmt);
+
+	if (cursor) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
+			TrackerProperty *domain_index;
+			const gchar *domain_index_uri;
+
+			domain_index_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
+			domain_index = tracker_ontologies_get_property_by_uri (domain_index_uri);
+			tracker_class_add_domain_index (class, domain_index);
+			tracker_property_add_domain_index (domain_index, class);
+		}
+
+		g_object_unref (cursor);
+	}
+}
+
 static void
 property_add_super_properties_from_db (TrackerDBInterface *iface,
                                        TrackerProperty *property)
@@ -1250,7 +1502,7 @@ property_add_super_properties_from_db (TrackerDBInterface *iface,
 	g_object_unref (stmt);
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
 			TrackerProperty *super_property;
 			const gchar *super_property_uri;
 
@@ -1269,6 +1521,8 @@ db_get_static_data (TrackerDBInterface *iface)
 	TrackerDBStatement *stmt;
 	TrackerDBCursor *cursor = NULL;
 	TrackerDBResultSet *result_set;
+	TrackerClass **classes;
+	guint n_classes, i;
 	GError *error = NULL;
 
 	stmt = tracker_db_interface_create_statement (iface, &error,
@@ -1282,7 +1536,7 @@ db_get_static_data (TrackerDBInterface *iface)
 	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, &error)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			TrackerOntology *ontology;
 			const gchar     *uri;
 			time_t           last_mod;
@@ -1320,7 +1574,7 @@ db_get_static_data (TrackerDBInterface *iface)
 	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, &error)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			TrackerNamespace *namespace;
 			const gchar      *uri, *prefix;
 
@@ -1359,7 +1613,7 @@ db_get_static_data (TrackerDBInterface *iface)
 	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, &error)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			TrackerClass *class;
 			const gchar  *uri;
 			gint          id;
@@ -1389,12 +1643,17 @@ db_get_static_data (TrackerDBInterface *iface)
 
 			class_add_super_classes_from_db (iface, class);
 
+			/* We do this later, we first need to load the properties too
+			   class_add_domain_indexes_from_db (iface, class); */
+
 			tracker_ontologies_add_class (class);
 			tracker_ontologies_add_id_uri_pair (id, uri);
 			tracker_class_set_id (class, id);
 
 			/* xsd classes do not derive from rdfs:Resource and do not use separate tables */
-			if (!g_str_has_prefix (tracker_class_get_name (class), "xsd:")) {
+			/* no need to fetch resource count if we are running in read-only mode (direct access) */
+			if (!g_str_has_prefix (tracker_class_get_name (class), "xsd:") &&
+			    (tracker_db_manager_get_flags () & TRACKER_DB_MANAGER_READONLY) == 0) {
 				/* update statistics */
 				stmt = tracker_db_interface_create_statement (iface, &error, "SELECT COUNT(1) FROM \"%s\"", tracker_class_get_name (class));
 
@@ -1445,7 +1704,7 @@ db_get_static_data (TrackerDBInterface *iface)
 	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, &error)) {
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			GValue value = { 0 };
 			TrackerProperty *property;
 			const gchar     *uri, *domain_uri, *range_uri, *secondary_index_uri, *default_value;
@@ -1549,6 +1808,7 @@ db_get_static_data (TrackerDBInterface *iface)
 
 			default_value = tracker_db_cursor_get_string (cursor, 13, NULL);
 
+			tracker_property_set_is_new_domain_index (property, FALSE);
 			tracker_property_set_is_new (property, FALSE);
 			tracker_property_set_transient (property, transient);
 			tracker_property_set_uri (property, uri);
@@ -1583,6 +1843,12 @@ db_get_static_data (TrackerDBInterface *iface)
 		cursor = NULL;
 	}
 
+	/* Now that the properties are loaded we can do this foreach class */
+	classes = tracker_ontologies_get_classes (&n_classes);
+	for (i = 0; i < n_classes; i++) {
+		class_add_domain_indexes_from_db (iface, classes[i]);
+	}
+
 	if (error) {
 		g_warning ("%s", error->message);
 		g_error_free (error);
@@ -1707,6 +1973,7 @@ create_decomposed_metadata_property_table (TrackerDBInterface *iface,
 	}
 
 	if (!in_update || (in_update && (tracker_property_get_is_new (property) ||
+	                                 tracker_property_get_is_new_domain_index (property) ||
 	                                 tracker_property_get_db_schema_changed (property)))) {
 		if (transient || tracker_property_get_multiple_values (property)) {
 			GString *sql;
@@ -1722,10 +1989,16 @@ create_decomposed_metadata_property_table (TrackerDBInterface *iface,
 			}
 
 			if (in_change && !tracker_property_get_is_new (property)) {
+				g_debug ("Drop index: DROP INDEX IF EXISTS \"%s_%s_ID\"\nRename: ALTER TABLE \"%s_%s\" RENAME TO \"%s_%s_TEMP\"",
+				         service_name, field_name, service_name, field_name,
+				         service_name, field_name);
+
 				tracker_db_interface_execute_query (iface, NULL,
 				                                    "DROP INDEX IF EXISTS \"%s_%s_ID\"",
 				                                    service_name,
 				                                    field_name);
+
+
 				tracker_db_interface_execute_query (iface, &error,
 				                                    "ALTER TABLE \"%s_%s\" RENAME TO \"%s_%s_TEMP\"",
 				                                    service_name, field_name, service_name, field_name);
@@ -1836,6 +2109,80 @@ create_decomposed_metadata_property_table (TrackerDBInterface *iface,
 
 }
 
+static gboolean
+is_a_domain_index (TrackerProperty **domain_indexes, TrackerProperty *property)
+{
+	while (*domain_indexes) {
+
+		if (*domain_indexes == property) {
+			return TRUE;
+		}
+
+		domain_indexes++;
+	}
+
+	return FALSE;
+}
+
+static void
+copy_from_domain_to_domain_index (TrackerDBInterface *iface,
+                                  TrackerProperty    *domain_index,
+                                  const gchar        *column_name,
+                                  const gchar        *column_suffix,
+                                  TrackerClass       *dest_domain)
+{
+	GError *error = NULL;
+	TrackerClass *source_domain;
+	const gchar *source_name, *dest_name;
+	gchar *query;
+
+	source_domain = tracker_property_get_domain (domain_index);
+	source_name = tracker_class_get_name (source_domain);
+	dest_name = tracker_class_get_name (dest_domain);
+
+	query = g_strdup_printf ("UPDATE \"%s\" SET \"%s%s\"=("
+	                         "SELECT \"%s%s\" FROM \"%s\" "
+	                         "WHERE \"%s\".ID = \"%s\".ID)",
+	                         dest_name,
+	                         column_name,
+	                         column_suffix ? column_suffix : "",
+	                         column_name,
+	                         column_suffix ? column_suffix : "",
+	                         source_name,
+	                         source_name,
+	                         dest_name);
+
+	g_debug ("Copying: '%s'", query);
+
+	tracker_db_interface_execute_query (iface, &error, "%s", query);
+
+	if (error) {
+		g_critical ("Ontology change failed while altering SQL table '%s'", error->message);
+		g_clear_error (&error);
+	}
+
+	g_free (query);
+}
+
+typedef struct {
+	TrackerProperty *prop;
+	const gchar *field_name;
+	const gchar *suffix;
+} ScheduleCopy;
+
+static void
+schedule_copy (GPtrArray *schedule,
+               TrackerProperty *prop,
+               const gchar *field_name,
+               const gchar *suffix)
+{
+	ScheduleCopy *sched = g_new0 (ScheduleCopy, 1);
+	sched->prop = prop;
+	sched->field_name = field_name,
+	sched->suffix = suffix;
+	g_ptr_array_add (schedule, sched);
+}
+
 static void
 create_decomposed_metadata_tables (TrackerDBInterface *iface,
                                    TrackerClass       *service,
@@ -1846,14 +2193,20 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 	GString          *create_sql = NULL;
 	GString          *in_col_sql = NULL;
 	GString          *sel_col_sql = NULL;
-	TrackerProperty **properties, *property;
+	TrackerProperty **properties, *property, **domain_indexes;
 	GSList           *class_properties, *field_it;
 	gboolean          main_class;
 	gint              i, n_props;
 	gboolean          in_alter = in_update;
 	GError           *error = NULL;
+	GPtrArray        *copy_schedule = NULL;
+
+	g_return_if_fail (TRACKER_IS_CLASS (service));
 
 	service_name = tracker_class_get_name (service);
+
+	g_return_if_fail (service_name != NULL);
+
 	main_class = (strcmp (service_name, "rdfs:Resource") == 0);
 
 	if (g_str_has_prefix (service_name, "xsd:")) {
@@ -1862,6 +2215,7 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 	}
 
 	if (in_change) {
+		g_debug ("Rename: ALTER TABLE \"%s\" RENAME TO \"%s_TEMP\"", service_name, service_name);
 		tracker_db_interface_execute_query (iface, &error, "ALTER TABLE \"%s\" RENAME TO \"%s_TEMP\"", service_name, service_name);
 		in_col_sql = g_string_new ("ID");
 		sel_col_sql = g_string_new ("ID");
@@ -1888,12 +2242,17 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 	}
 
 	properties = tracker_ontologies_get_properties (&n_props);
+	domain_indexes = tracker_class_get_domain_indexes (service);
+
 	class_properties = NULL;
 
 	for (i = 0; i < n_props; i++) {
-		property = properties[i];
+		gboolean is_domain_index;
 
-		if (tracker_property_get_domain (property) == service) {
+		property = properties[i];
+		is_domain_index = is_a_domain_index (domain_indexes, property);
+
+		if (tracker_property_get_domain (property) == service || is_domain_index) {
 			gboolean put_change;
 			const gchar *sql_type_for_single_value = NULL;
 			const gchar *field_name;
@@ -1929,6 +2288,14 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 					                        field_name,
 					                        sql_type_for_single_value);
 
+					if (!copy_schedule) {
+						copy_schedule = g_ptr_array_new_with_free_func (g_free);
+					}
+
+					if (is_domain_index && tracker_property_get_is_new_domain_index (property)) {
+						schedule_copy (copy_schedule, property, field_name, NULL);
+					}
+
 					/* add DEFAULT in case that the ontology specifies a default value,
 					   assumes that default values never contain quotes */
 					if (default_value != NULL) {
@@ -1942,15 +2309,26 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 					g_string_append_printf (create_sql, ", \"%s:graph\" INTEGER",
 					                        field_name);
 
+					if (is_domain_index && tracker_property_get_is_new_domain_index (property)) {
+						schedule_copy (copy_schedule, property, field_name, ":graph");
+					}
+
 					if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
 						/* xsd:dateTime is stored in three columns:
 						 * universal time, local date, local time of day */
 						g_string_append_printf (create_sql, ", \"%s:localDate\" INTEGER, \"%s:localTime\" INTEGER",
-							                tracker_property_get_name (property),
-							                tracker_property_get_name (property));
+						                        tracker_property_get_name (property),
+						                        tracker_property_get_name (property));
+
+						if (is_domain_index && tracker_property_get_is_new_domain_index (property)) {
+							schedule_copy (copy_schedule, property, field_name, ":localTime");
+							schedule_copy (copy_schedule, property, field_name, ":localDate");
+						}
+
 					}
 
-				} else if (tracker_property_get_is_new (property)) {
+				} else if ((!is_domain_index && tracker_property_get_is_new (property)) ||
+				           (is_domain_index && tracker_property_get_is_new_domain_index (property))) {
 					GString *alter_sql = NULL;
 
 					put_change = FALSE;
@@ -1970,22 +2348,38 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 					if (tracker_property_get_is_inverse_functional_property (property)) {
 						g_string_append (alter_sql, " UNIQUE");
 					}
+					g_debug ("Altering: '%s'", alter_sql->str);
 					tracker_db_interface_execute_query (iface, &error, "%s", alter_sql->str);
 					if (error) {
 						g_critical ("Ontology change failed while altering SQL table '%s'", error->message);
 						g_clear_error (&error);
+					} else if (is_domain_index) {
+						copy_from_domain_to_domain_index (iface, property,
+						                                  field_name, NULL,
+						                                  service);
+						/* This is implicit for all domain-specific-indices */
+						set_index_for_single_value_property (iface, service_name,
+						                                     field_name, TRUE);
 					}
+
 					g_string_free (alter_sql, TRUE);
 
+
 					alter_sql = g_string_new ("ALTER TABLE ");
 					g_string_append_printf (alter_sql, "\"%s\" ADD COLUMN \"%s:graph\" INTEGER",
 					                        service_name,
 					                        field_name);
+					g_debug ("Altering: '%s'", alter_sql->str);
 					tracker_db_interface_execute_query (iface, &error, "%s", alter_sql->str);
 					if (error) {
 						g_critical ("Ontology change failed while altering SQL table '%s'", error->message);
 						g_clear_error (&error);
+					} else if (is_domain_index) {
+						copy_from_domain_to_domain_index (iface, property,
+						                                  field_name, ":graph",
+						                                  service);
 					}
+
 					g_string_free (alter_sql, TRUE);
 
 					if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
@@ -1993,22 +2387,35 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 						g_string_append_printf (alter_sql, "\"%s\" ADD COLUMN \"%s:localDate\" INTEGER",
 						                        service_name,
 						                        field_name);
+						g_debug ("Altering: '%s'", alter_sql->str);
 						tracker_db_interface_execute_query (iface, &error, "%s", alter_sql->str);
 						if (error) {
 							g_critical ("Ontology change failed while altering SQL table '%s'", error->message);
 							g_clear_error (&error);
+						}	else if (is_domain_index) {
+							copy_from_domain_to_domain_index (iface, property,
+							                                  field_name, ":localDate",
+							                                  service);
 						}
+
 						g_string_free (alter_sql, TRUE);
 
+
 						alter_sql = g_string_new ("ALTER TABLE ");
 						g_string_append_printf (alter_sql, "\"%s\" ADD COLUMN \"%s:localTime\" INTEGER",
 						                        service_name,
 						                        field_name);
+						g_debug ("Altering: '%s'", alter_sql->str);
 						tracker_db_interface_execute_query (iface, &error, "%s", alter_sql->str);
 						if (error) {
 							g_critical ("Ontology change failed while altering SQL table '%s'", error->message);
 							g_clear_error (&error);
+						} else if (is_domain_index) {
+							copy_from_domain_to_domain_index (iface, property,
+							                                  field_name, ":localTime",
+							                                  service);
 						}
+
 						g_string_free (alter_sql, TRUE);
 
 					}
@@ -2025,6 +2432,7 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 
 	if (create_sql) {
 		g_string_append (create_sql, ")");
+		g_debug ("Creating: '%s'", create_sql->str);
 		tracker_db_interface_execute_query (iface, NULL, "%s", create_sql->str);
 		g_string_free (create_sql, TRUE);
 	}
@@ -2032,12 +2440,17 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 	/* create index for single-valued fields */
 	for (field_it = class_properties; field_it != NULL; field_it = field_it->next) {
 		TrackerProperty *field, *secondary_index;
-		const char   *field_name;
+		const char *field_name;
+		gboolean is_domain_index;
 
 		field = field_it->data;
 
+		/* This is implicit for all domain-specific-indices */
+		is_domain_index = is_a_domain_index (domain_indexes, field);
+
 		if (!tracker_property_get_multiple_values (field)
-		    && tracker_property_get_indexed (field)) {
+		    && (tracker_property_get_indexed (field) || is_domain_index)) {
+
 			field_name = tracker_property_get_name (field);
 
 			secondary_index = tracker_property_get_secondary_index (field);
@@ -2053,7 +2466,6 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 
 	g_slist_free (class_properties);
 
-
 	if (in_change && sel_col_sql && in_col_sql) {
 		gchar *query;
 		GError *error = NULL;
@@ -2063,12 +2475,15 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 		                         service_name, in_col_sql->str,
 		                         sel_col_sql->str, service_name);
 
+		g_debug ("Copy: %s", query);
+
 		tracker_db_interface_execute_query (iface, &error, "%s", query);
 		if (error) {
 			g_critical ("Ontology change failed while merging SQL table data '%s'", error->message);
 			g_clear_error (&error);
 		}
 		g_free (query);
+		g_debug ("Rename (drop): DROP TABLE \"%s_TEMP\"", service_name);
 		tracker_db_interface_execute_query (iface, &error, "DROP TABLE \"%s_TEMP\"", service_name);
 		if (error) {
 			g_critical ("Ontology change failed while dropping SQL table '%s'", error->message);
@@ -2081,6 +2496,16 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 	if (sel_col_sql)
 		g_string_free (sel_col_sql, TRUE);
 
+	if (copy_schedule) {
+		guint i;
+		for (i = 0; i < copy_schedule->len; i++) {
+			ScheduleCopy *sched = g_ptr_array_index (copy_schedule, i);
+			copy_from_domain_to_domain_index (iface, sched->prop,
+			                                  sched->field_name, sched->suffix,
+			                                  service);
+		}
+		g_ptr_array_free (copy_schedule, TRUE);
+	}
 }
 
 static void
@@ -2128,6 +2553,7 @@ tracker_data_ontology_import_finished (void)
 	}
 
 	for (i = 0; i < n_props; i++) {
+		tracker_property_set_is_new_domain_index (properties[i], FALSE);
 		tracker_property_set_is_new (properties[i], FALSE);
 		tracker_property_set_db_schema_changed (properties[i], FALSE);
 	}
@@ -2228,14 +2654,14 @@ get_new_service_id (TrackerDBInterface *iface)
 	}
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor, &error);
-		max_service_id = tracker_db_cursor_get_int (cursor, 0);
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
+			max_service_id = tracker_db_cursor_get_int (cursor, 0);
+		}
 		g_object_unref (cursor);
 	}
 
 	if (error) {
-		g_warning ("%s", error->message);
-		g_error_free (error);
+		g_error ("Unable to get max ID, aborting: %s", error->message);
 	}
 
 	return ++max_service_id;
@@ -2243,7 +2669,7 @@ get_new_service_id (TrackerDBInterface *iface)
 
 gboolean
 tracker_data_manager_init (TrackerDBManagerFlags  flags,
-                           const gchar          **test_schemas,
+                          const gchar          **test_schemas,
                            gboolean              *first_time,
                            gboolean               journal_check,
                            TrackerBusyCallback    busy_callback,
@@ -2258,9 +2684,12 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 	GList *sorted = NULL, *l;
 	const gchar *env_path;
 	gint max_id = 0;
+	gboolean read_only;
 
 	tracker_data_update_init ();
 
+	read_only = (flags & TRACKER_DB_MANAGER_READONLY) ? TRUE : FALSE;
+
 	/* First set defaults for return values */
 	if (first_time) {
 		*first_time = FALSE;
@@ -2275,7 +2704,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 	read_journal = FALSE;
 
-	if (!tracker_db_manager_init (flags, &is_first_time_index, TRUE)) {
+	if (!tracker_db_manager_init (flags, &is_first_time_index, FALSE)) {
 		return FALSE;
 	}
 
@@ -2322,13 +2751,13 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		load_ontology_from_journal (&classes, &properties, &id_uri_map);
 
 		/* Read first ontology and commit it into the DB */
-		tracker_data_begin_db_transaction_for_replay (tracker_db_journal_reader_get_time ());
+		tracker_data_begin_transaction_for_replay (tracker_db_journal_reader_get_time (), NULL);
 
 		/* This is a no-op when FTS is disabled */
 		tracker_db_interface_sqlite_fts_init (iface, TRUE);
 
 		tracker_data_ontology_import_into_db (FALSE);
-		tracker_data_commit_db_transaction ();
+		tracker_data_commit_transaction (NULL);
 		tracker_db_journal_reader_shutdown ();
 
 		/* Start replay. Ontology changes might happen during replay of the journal. */
@@ -2347,7 +2776,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		g_hash_table_unref (properties);
 		g_hash_table_unref (id_uri_map);
 
-	} else if (is_first_time_index) {
+	} else if (is_first_time_index && !read_only) {
 		sorted = get_ontologies (test_schemas != NULL, ontologies_dir);
 
 		/* Truncate journal as it does not even contain a single valid transaction
@@ -2377,10 +2806,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 			}
 		}
 
-		tracker_data_begin_db_transaction ();
-
-		/* Not an ontology transaction: this is the first ontology */
-		tracker_db_journal_start_transaction (time (NULL));
+		tracker_data_begin_transaction (NULL);
 
 		/* This is a no-op when FTS is disabled */
 		tracker_db_interface_sqlite_fts_init (iface, TRUE);
@@ -2405,8 +2831,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 			}
 		}
 
-		tracker_db_journal_commit_db_transaction ();
-		tracker_data_commit_db_transaction ();
+		tracker_data_commit_transaction (NULL);
 
 		g_list_foreach (sorted, (GFunc) g_free, NULL);
 		g_list_free (sorted);
@@ -2426,7 +2851,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		tracker_db_interface_sqlite_fts_init (iface, FALSE);
 	}
 
-	if (check_ontology) {
+	if (check_ontology && !read_only) {
 		GList *to_reload = NULL;
 		GList *ontos = NULL;
 		guint p;
@@ -2459,7 +2884,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 		/* check ontology against database */
 
-		tracker_data_begin_db_transaction ();
+		tracker_data_begin_transaction (NULL);
 
 		/* This _is_ an ontology transaction, it represents a change to the
 		 * ontology. We mark it up as such in the journal, so that replay_journal
@@ -2489,7 +2914,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		                                     NULL);
 
 		if (cursor) {
-			while (tracker_db_cursor_iter_next (cursor, &error)) {
+			while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 				const gchar *onto_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 				/* It's stored as an int in the db anyway. This is caused by
 				 * string_to_gvalue in tracker-data-update.c */
@@ -2592,10 +3017,12 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		}
 
 		if (to_reload) {
+			tracker_data_ontology_process_changes_pre_db (seen_classes, seen_properties);
+
 			/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
 			tracker_data_ontology_import_into_db (TRUE);
 
-			tracker_data_ontology_process_changes (seen_classes, seen_properties);
+			tracker_data_ontology_process_changes_post_db (seen_classes, seen_properties);
 
 			for (l = to_reload; l; l = l->next) {
 				const gchar *ontology_path = l->data;
@@ -2603,16 +3030,18 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 				import_ontology_path (ontology_path, TRUE, !journal_check);
 			}
 			g_list_free (to_reload);
+
+			tracker_data_ontology_process_changes_post_import (seen_classes, seen_properties);
 		}
 
+
 		tracker_data_ontology_free_seen (seen_classes);
 		tracker_data_ontology_free_seen (seen_properties);
 
 		/* Reset the is_new flag for all classes and properties */
 		tracker_data_ontology_import_finished ();
 
-		tracker_db_journal_commit_db_transaction ();
-		tracker_data_commit_db_transaction ();
+		tracker_data_commit_transaction (NULL);
 
 		g_hash_table_unref (ontos_table);
 
@@ -2624,6 +3053,10 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 	g_free (ontologies_dir);
 
+	if (busy_callback) {
+		busy_callback ("Idle", 1, busy_user_data);
+	}
+
 	return TRUE;
 }
 
@@ -2641,90 +3074,3 @@ tracker_data_manager_shutdown (void)
 
 	initialized = FALSE;
 }
-
-gint64
-tracker_data_manager_get_db_option_int64 (const gchar *option)
-{
-	TrackerDBInterface *iface;
-	TrackerDBStatement *stmt;
-	TrackerDBResultSet *result_set = NULL;
-	gchar              *str;
-	gint                value = 0;
-	GError             *error = NULL;
-
-	g_return_val_if_fail (option != NULL, 0);
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	stmt = tracker_db_interface_create_statement (iface, &error, "SELECT OptionValue FROM Options WHERE OptionKey = ?");
-
-	if (stmt) {
-		tracker_db_statement_bind_text (stmt, 0, option);
-		result_set = tracker_db_statement_execute (stmt, &error);
-		g_object_unref (stmt);
-	}
-
-	if (result_set) {
-		tracker_db_result_set_get (result_set, 0, &str, -1);
-
-		if (str) {
-			value = g_ascii_strtoull (str, NULL, 10);
-			g_free (str);
-		}
-
-		g_object_unref (result_set);
-	}
-
-	if (error) {
-		g_warning ("%s", error->message);
-		g_error_free (error);
-		return -1;
-	}
-
-	return value;
-}
-
-void
-tracker_data_manager_set_db_option_int64 (const gchar *option,
-                                          gint64       value)
-{
-	TrackerDBInterface *iface;
-	TrackerDBStatement *stmt;
-	gchar              *str;
-	GError             *error = NULL;
-
-	g_return_if_fail (option != NULL);
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	stmt = tracker_db_interface_create_statement (iface, &error, "REPLACE INTO Options (OptionKey, OptionValue) VALUES (?,?)");
-
-	if (stmt) {
-		tracker_db_statement_bind_text (stmt, 0, option);
-
-		str = g_strdup_printf ("%"G_GINT64_FORMAT, value);
-		tracker_db_statement_bind_text (stmt, 1, str);
-		g_free (str);
-
-		tracker_db_statement_execute (stmt, &error);
-		g_object_unref (stmt);
-	}
-
-	if (error) {
-		g_warning ("%s", error->message);
-		g_error_free (error);
-		return;
-	}
-}
-
-gboolean
-tracker_data_manager_interrupt_thread (GThread *thread)
-{
-	return tracker_db_manager_interrupt_thread (thread);
-}
-
-void
-tracker_data_manager_interrupt_thread_reset (GThread *thread)
-{
-	tracker_db_manager_interrupt_thread_reset (thread);
-}
