@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,19 +20,23 @@
 #ifndef __LIBTRACKER_MINER_DBUS_H__
 #define __LIBTRACKER_MINER_DBUS_H__
 
+#if !defined (__LIBTRACKER_MINER_H_INSIDE__) && !defined (TRACKER_COMPILATION)
+#error "Only <libtracker-miner/tracker-miner.h> can be included directly."
+#endif
+
 #include <libtracker-common/tracker-dbus.h>
 
-#include "tracker-miner.h"
+#include "tracker-miner-object.h"
 
 typedef void (* TrackerMinerDBusNameFunc) (TrackerMiner *miner,
                                            const gchar  *name,
                                            gboolean      available);
 
-void tracker_miner_dbus_init           (TrackerMiner             *miner,
-                                        const DBusGObjectInfo    *info);
-void tracker_miner_dbus_shutdown       (TrackerMiner             *miner);
-void tracker_miner_dbus_add_name_watch (TrackerMiner             *miner,
-                                        const gchar              *name,
-                                        TrackerMinerDBusNameFunc  func);
+void _tracker_miner_dbus_init           (TrackerMiner             *miner,
+                                         const DBusGObjectInfo    *info);
+void _tracker_miner_dbus_shutdown       (TrackerMiner             *miner);
+void _tracker_miner_dbus_add_name_watch (TrackerMiner             *miner,
+                                         const gchar              *name,
+                                         TrackerMinerDBusNameFunc  func);
 
 #endif /* __LIBTRACKER_MINER_DBUS_H__ */
