@@ -30,13 +30,19 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
+#include <gio/gunixinputstream.h>
+
+#include <dbus/dbus-glib-lowlevel.h>
+#include <dbus/dbus.h>
+
 #include <libtracker-common/tracker-date-time.h>
+#include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-power.h>
 #include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
-#include <libtracker-db/tracker-db.h>
+#include <libtracker-data/tracker-db-manager.h>
 
 #include "tracker-miner-files.h"
 #include "tracker-config.h"
@@ -52,6 +58,9 @@
  */
 #define N_DAYS_THRESHOLD 3
 
+/* Default DBus timeout to be used in requests to extractor (milliseconds) */
+#define EXTRACTOR_DBUS_TIMEOUT 60000
+
 #define TRACKER_MINER_FILES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER_FILES, TrackerMinerFilesPrivate))
 
 static GQuark miner_files_error_quark = 0;
@@ -63,9 +72,18 @@ struct ProcessFileData {
 	TrackerSparqlBuilder *sparql;
 	GCancellable *cancellable;
 	GFile *file;
+#ifdef HAVE_DBUS_FD_PASSING
+	DBusPendingCall *call;
+#else /* HAVE_DBUS_FD_PASSING */
 	DBusGProxyCall *call;
+#endif /* HAVE_DBUS_FD_PASSING */
 };
 
+typedef struct {
+	org_freedesktop_Tracker1_Extract_get_metadata_reply callback;
+	gpointer user_data;
+} FastAsyncData;
+
 struct TrackerMinerFilesPrivate {
 	TrackerConfig *config;
 	TrackerStorage *storage;
@@ -85,6 +103,7 @@ struct TrackerMinerFilesPrivate {
 #endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
 	gulong finished_handler;
 
+	DBusGConnection *connection;
 	DBusGProxy *extractor_proxy;
 
 	GQuark quark_mount_point_uuid;
@@ -160,7 +179,7 @@ static void        trigger_recheck_cb                   (GObject              *g
 static void        index_volumes_changed_cb             (GObject              *gobject,
 							 GParamSpec           *arg1,
 							 gpointer              user_data);
-static DBusGProxy *extractor_create_proxy               (void);
+static DBusGProxy *extractor_create_proxy               (DBusGConnection      *connection);
 static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
                                                          GFile                *file);
 static gboolean    miner_files_check_directory          (TrackerMinerFS       *fs,
@@ -238,6 +257,7 @@ static void
 tracker_miner_files_init (TrackerMinerFiles *mf)
 {
 	TrackerMinerFilesPrivate *priv;
+	GError *error = NULL;
 
 	priv = mf->private = TRACKER_MINER_FILES_GET_PRIVATE (mf);
 
@@ -272,7 +292,15 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	                  mf);
 
 	/* Set up extractor and signals */
-	priv->extractor_proxy = extractor_create_proxy ();
+	priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+
+	if (!priv->connection) {
+		g_critical ("Could not connect to the D-Bus session bus, %s",
+		            error ? error->message : "no error given.");
+		g_error_free (error);
+	}
+
+	priv->extractor_proxy = extractor_create_proxy (priv->connection);
 
 	priv->quark_mount_point_uuid = g_quark_from_static_string ("tracker-mount-point-uuid");
 	priv->quark_directory_config_root = g_quark_from_static_string ("tracker-directory-config-root");
@@ -581,17 +609,60 @@ miner_files_constructed (GObject *object)
 	disk_space_check_start (mf);
 }
 
+static void
+ensure_mount_point_exists (TrackerMinerFiles *miner,
+                           GFile             *mount_point,
+                           GString           *accumulator)
+{
+	gchar *iri;
+	gchar *uri;
+
+	uri = g_file_get_uri (mount_point);
+
+	/* Query the store for the URN of the mount point */
+	iri = tracker_miner_fs_query_urn (TRACKER_MINER_FS (miner),
+	                                  mount_point);
+
+	if (iri) {
+		/* If exists, just return, nothing else to do */
+		g_message ("Mount point '%s' already exists in store: '%s'",
+		           uri, iri);
+		g_free (iri);
+	} else {
+		/* If it doesn't exist, we need to create it */
+		g_message ("Mount point '%s' does not exist in store, need to create it",
+		           uri);
+
+		/* Create a nfo:Folder for the mount point */
+		g_string_append_printf (accumulator,
+		                        "INSERT SILENT INTO <" TRACKER_MINER_FS_GRAPH_URN "> {"
+		                        " _:file a nfo:FileDataObject, nie:InformationElement, nfo:Folder ; "
+		                        "        nie:isStoredAs _:file ; "
+		                        "        nie:url \"%s\" ; "
+		                        "        nie:mimeType \"inode/directory\" ; "
+		                        "        nfo:fileLastModified \"1981-06-05T02:20:00Z\" . "
+		                        "}",
+		                        uri);
+
+		/* Tell the underlying miner-fs that we created a directory without
+		 * a valid specific parent */
+		tracker_miner_fs_add_directory_without_parent (TRACKER_MINER_FS (miner),
+		                                               mount_point);
+	}
+
+	g_free (uri);
+}
+
 static void
 set_up_mount_point_cb (GObject      *source,
                        GAsyncResult *result,
                        gpointer      user_data)
 {
+	TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (source);
 	gchar *removable_device_urn = user_data;
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (source),
-	                                     result,
-	                                     &error);
+	tracker_sparql_connection_update_finish (connection, result, &error);
 
 	if (error) {
 		g_critical ("Could not set mount point in database '%s', %s",
@@ -643,12 +714,13 @@ set_up_mount_point (TrackerMinerFiles *miner,
 {
 	GString *queries;
 
-	g_debug ("Mount point state being set in DB for URN '%s'",
-	         removable_device_urn);
-
 	queries = g_string_new (NULL);
 
 	if (mounted) {
+		g_debug ("Mount point state (MOUNTED) being set in DB for URN '%s' (mount_point: %s)",
+		         removable_device_urn,
+		         mount_point ? mount_point : "unknown");
+
 		if (mount_point) {
 			GFile *file;
 			gchar *uri;
@@ -656,6 +728,10 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			file = g_file_new_for_path (mount_point);
 			uri = g_file_get_uri (file);
 
+			/* Before assigning a nfo:FileDataObject as tracker:mountPoint for
+			 * the volume, make sure the nfo:FileDataObject exists in the store */
+			ensure_mount_point_exists (miner, file, queries);
+
 			g_string_append_printf (queries,
 			                        "DELETE FROM <%s> { "
 			                        "  <%s> tracker:mountPoint ?u "
@@ -694,6 +770,9 @@ set_up_mount_point (TrackerMinerFiles *miner,
 	} else {
 		gchar *now;
 
+		g_debug ("Mount point state (UNMOUNTED) being set in DB for URN '%s'",
+		         removable_device_urn);
+
 		now = tracker_date_to_string (time (NULL));
 
 		g_string_append_printf (queries,
@@ -721,11 +800,12 @@ set_up_mount_point (TrackerMinerFiles *miner,
 	if (accumulator) {
 		g_string_append_printf (accumulator, "%s ", queries->str);
 	} else {
-		tracker_miner_execute_update (TRACKER_MINER (miner),
-		                              queries->str,
-		                              NULL,
-		                              set_up_mount_point_cb,
-		                              g_strdup (removable_device_urn));
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                        queries->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        set_up_mount_point_cb,
+                                                        g_strdup (removable_device_urn));
 	}
 
 	g_string_free (queries, TRUE);
@@ -738,7 +818,7 @@ init_mount_points_cb (GObject      *source,
 {
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (source),
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (source),
 	                                     result,
 	                                     &error);
 
@@ -751,28 +831,29 @@ init_mount_points_cb (GObject      *source,
 		 * the mount points, as we need the correct tracker:isMounted value
 		 * for all currently mounted volumes
 		 */
-		init_stale_volume_removal (TRACKER_MINER_FILES (source));
+		init_stale_volume_removal (TRACKER_MINER_FILES (user_data));
 	}
 }
 
 static void
-query_mount_points_cb (GObject      *source,
-                       GAsyncResult *result,
-                       gpointer      user_data)
+init_mount_points (TrackerMinerFiles *miner_files)
 {
-	TrackerMiner *miner = TRACKER_MINER (source);
+	TrackerMiner *miner = TRACKER_MINER (miner_files);
 	TrackerMinerFilesPrivate *priv;
 	GHashTable *volumes;
 	GHashTableIter iter;
 	gpointer key, value;
 	GString *accumulator;
 	GError *error = NULL;
-	TrackerResultIterator *iterator;
+	TrackerSparqlCursor *cursor;
 	GSList *uuids, *u;
 
-	iterator = tracker_miner_execute_sparql_finish (miner,
-	                                                result,
-	                                                &error);
+	g_debug ("Initializing mount points...");
+
+	/* First, get all mounted volumes, according to tracker-store (SYNC!) */
+	cursor = tracker_sparql_connection_query (tracker_miner_get_connection (miner),
+	                                          "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
+	                                          NULL, &error);
 	if (error) {
 		g_critical ("Could not obtain the mounted volumes: %s", error->message);
 		g_error_free (error);
@@ -792,13 +873,13 @@ query_mount_points_cb (GObject      *source,
                              g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
                              GINT_TO_POINTER (VOLUME_MOUNTED));
 
-	while (tracker_result_iterator_next (iterator)) {
+	while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 		gint state;
 		const gchar *urn;
 
 		state = VOLUME_MOUNTED_IN_STORE;
 
-		urn = tracker_result_iterator_value (iterator, 0);
+		urn = tracker_sparql_cursor_get_string (cursor, 0, NULL);
 
 		if (strcmp (urn, TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN) == 0) {
 			/* Report non-removable media to be mounted by HAL as well */
@@ -808,6 +889,8 @@ query_mount_points_cb (GObject      *source,
 		g_hash_table_replace (volumes, g_strdup (urn), GINT_TO_POINTER (state));
 	}
 
+	g_object_unref (cursor);
+
 	/* Then, get all currently mounted non-REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, 0, TRUE);
 	for (u = uuids; u; u = u->next) {
@@ -824,6 +907,9 @@ query_mount_points_cb (GObject      *source,
 		g_hash_table_replace (volumes, non_removable_device_urn, GINT_TO_POINTER (state));
 	}
 
+	g_slist_foreach (uuids, (GFunc) g_free, NULL);
+	g_slist_free (uuids);
+
 	/* Then, get all currently mounted REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, TRACKER_STORAGE_REMOVABLE, FALSE);
 	for (u = uuids; u; u = u->next) {
@@ -909,11 +995,12 @@ query_mount_points_cb (GObject      *source,
 	}
 
 	if (accumulator->str[0] != '\0') {
-		tracker_miner_execute_update (miner,
-		                              accumulator->str,
-		                              NULL,
-		                              init_mount_points_cb,
-		                              NULL);
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (miner),
+                                                        accumulator->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        init_mount_points_cb,
+                                                        miner);
 	} else {
 		/* If no further mount point initialization was needed,
 		 * initialize stale volume removal here. */
@@ -924,20 +1011,6 @@ query_mount_points_cb (GObject      *source,
 	g_hash_table_unref (volumes);
 }
 
-static void
-init_mount_points (TrackerMinerFiles *miner)
-{
-	g_debug ("Initializing mount points");
-
-	/* First, get all mounted volumes, according to tracker-store */
-	tracker_miner_execute_sparql (TRACKER_MINER (miner),
-	                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
-	                              NULL,
-	                              query_mount_points_cb,
-	                              NULL);
-}
-
-
 static gboolean
 cleanup_stale_removable_volumes_cb (gpointer user_data)
 {
@@ -1087,11 +1160,12 @@ mount_point_added_cb (TrackerStorage *storage,
 	queries = g_string_new ("");
 	set_up_mount_point (miner, urn, mount_point, TRUE, queries);
 	set_up_mount_point_type (miner, urn, removable, optical, queries);
-	tracker_miner_execute_update (TRACKER_MINER (miner),
-	                              queries->str,
-	                              NULL,
-	                              set_up_mount_point_cb,
-	                              g_strdup (urn));
+	tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                queries->str,
+                                                G_PRIORITY_DEFAULT,
+                                                NULL,
+                                                set_up_mount_point_cb,
+                                                g_strdup (urn));
 	g_string_free (queries, TRUE);
 	g_free (urn);
 }
@@ -1200,6 +1274,8 @@ index_on_battery_cb (GObject    *object,
 	check_battery_status (mf);
 }
 
+#endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
+
 /* Called when mining has finished the first time */
 static void
 miner_finished_cb (TrackerMinerFS *fs,
@@ -1229,8 +1305,6 @@ miner_finished_cb (TrackerMinerFS *fs,
 #endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
 }
 
-#endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
-
 static void
 mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
                       GMount            *mount,
@@ -1772,20 +1846,11 @@ process_file_data_free (ProcessFileData *data)
 }
 
 static DBusGProxy *
-extractor_create_proxy (void)
+extractor_create_proxy (DBusGConnection *connection)
 {
 	DBusGProxy *proxy;
-	DBusGConnection *connection;
-	GError *error = NULL;
 
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-
-	if (!connection) {
-		g_critical ("Could not connect to the D-Bus session bus, %s",
-		            error ? error->message : "no error given.");
-		g_clear_error (&error);
-		return FALSE;
-	}
+	g_return_val_if_fail (connection, NULL);
 
 	/* Get proxy for the extractor */
 	proxy = dbus_g_proxy_new_for_name (connection,
@@ -1795,6 +1860,16 @@ extractor_create_proxy (void)
 
 	if (!proxy) {
 		g_critical ("Could not create a DBusGProxy to the extract service");
+	} else {
+		/* Set default timeout for DBus requests to be around 60s.
+		 * Assuming that the files which need more time to get extracted are PDFs
+		 * using libpoppler, we already have a limit in the PDF extractor not to
+		 * spend more than 5s extraction contents. And, assuming the default
+		 * value of 10 in process-pool-limit, it means we may end up queueing up
+		 * to 10 PDF files which may need 5s each, so in order not to have dbus
+		 * timeouts in this case, any value greater than 5*10 would be good.
+		 */
+		dbus_g_proxy_set_default_timeout (proxy, EXTRACTOR_DBUS_TIMEOUT);
 	}
 
 	return proxy;
@@ -1814,6 +1889,8 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 		/* Something bad happened, notify about the error */
 		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 		process_file_data_free (data);
+		/* Always free input GError. We want to behave exactly as if this
+		 * callback were one used in an async dbus-glib query.  */
 		g_error_free (error);
 		return;
 	}
@@ -1854,8 +1931,8 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 
 		removable_device_urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
 		uri = g_file_get_uri (G_FILE (data->file));
-
 		queries = g_string_new ("");
+
 		g_string_append_printf (queries,
 		                        "DELETE FROM <%s> { "
 		                        "  <%s> tracker:mountPoint ?unknown "
@@ -1885,8 +1962,11 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
 
 	process_file_data_free (data);
+#ifndef HAVE_DBUS_FD_PASSING
+	/* When using DBus FD passing, we let the caller free */
 	g_free (preupdate);
 	g_free (sparql);
+#endif /* HAVE_DBUS_FD_PASSING */
 }
 
 static void
@@ -1896,8 +1976,10 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	GError *error;
 
 	/* Cancel extractor call */
+#ifndef HAVE_DBUS_FD_PASSING
 	dbus_g_proxy_cancel_call (data->miner->private->extractor_proxy,
 	                          data->call);
+#endif /* HAVE_DBUS_FD_PASSING */
 
 	error = g_error_new_literal (miner_files_error_quark, 0, "Embedded metadata extraction was cancelled");
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
@@ -1906,16 +1988,127 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 	g_error_free (error);
 }
 
+#ifdef HAVE_DBUS_FD_PASSING
+static FastAsyncData*
+fast_async_data_new (org_freedesktop_Tracker1_Extract_get_metadata_reply callback,
+                     gpointer       user_data)
+{
+	FastAsyncData *data;
+
+	data = g_slice_new0 (FastAsyncData);
+	data->callback = callback;
+	data->user_data = user_data;
+
+	return data;
+}
+
+static void
+fast_async_data_free (FastAsyncData *data)
+{
+	g_slice_free (FastAsyncData, data);
+}
+
+static void
+get_metadata_fast_cb (void     *buffer,
+                      gssize    buffer_size,
+                      GError   *error,
+                      gpointer  user_data)
+{
+	FastAsyncData *data;
+	ProcessFileData *process_data;
+	gchar *preupdate;
+	gchar *sparql = NULL;
+
+	data = user_data;
+	process_data = data->user_data;
+
+	preupdate = buffer;
+	if (G_UNLIKELY (error)) {
+		if (error->code != G_IO_ERROR_CANCELLED) {
+			/* ProcessFileData and error are freed in the callback */
+			(* data->callback) (NULL, NULL, NULL, error, process_data);
+		} else {
+			/* Free error ourselves */
+			g_error_free (error);
+		}
+	} else {
+		if (buffer_size) {
+		/* sparql is stored just after preupdate in the original buffer */
+			sparql = preupdate + strlen (preupdate) + 1;
+		}
+
+		(* data->callback) (NULL, preupdate, sparql, NULL, data->user_data);
+		g_free (preupdate);
+	}
+
+	fast_async_data_free (data);
+}
+
+static void
+get_metadata_fast_async (DBusConnection  *connection,
+                         const gchar     *uri,
+                         const gchar     *mime_type,
+                         GCancellable    *cancellable,
+                         org_freedesktop_Tracker1_Extract_get_metadata_reply callback,
+                         ProcessFileData *user_data)
+{
+	int pipefd[2];
+	DBusMessage *message;
+	FastAsyncData *data;
+
+	g_return_if_fail (connection);
+	g_return_if_fail (uri);
+	g_return_if_fail (mime_type);
+	g_return_if_fail (callback);
+
+	if (pipe (pipefd) < 0) {
+		g_critical ("Coudln't open pipe");
+		return;
+	}
+
+	message = dbus_message_new_method_call (TRACKER_DBUS_SERVICE_EXTRACT,
+	                                        TRACKER_DBUS_PATH_EXTRACT,
+	                                        TRACKER_DBUS_INTERFACE_EXTRACT,
+	                                        "GetMetadataFast");
+	dbus_message_append_args (message,
+	                          DBUS_TYPE_STRING, &uri,
+	                          DBUS_TYPE_STRING, &mime_type,
+	                          DBUS_TYPE_UNIX_FD, &pipefd[1],
+	                          DBUS_TYPE_INVALID);
+	close (pipefd[1]);
+
+	data = fast_async_data_new (callback,
+	                            user_data);
+
+	tracker_dbus_send_and_splice_async (connection,
+	                                    message,
+	                                    pipefd[0],
+	                                    cancellable,
+	                                    get_metadata_fast_cb,
+	                                    data);
+}
+#endif /* HAVE_DBUS_FD_PASSING */
+
 static void
 extractor_get_embedded_metadata (ProcessFileData *data,
                                  const gchar     *uri,
                                  const gchar     *mime_type)
 {
+#ifdef HAVE_DBUS_FD_PASSING
+	get_metadata_fast_async (dbus_g_connection_get_connection (data->miner->private->connection),
+	                         uri,
+	                         mime_type,
+	                         data->cancellable,
+	                         extractor_get_embedded_metadata_cb,
+	                         data);
+	data->call = NULL;
+#else /* HAVE_DBUS_FD_PASSING */
 	data->call = org_freedesktop_Tracker1_Extract_get_metadata_async (data->miner->private->extractor_proxy,
 	                                                                  uri,
 	                                                                  mime_type,
 	                                                                  extractor_get_embedded_metadata_cb,
 	                                                                  data);
+#endif
 	g_signal_connect (data->cancellable, "cancelled",
 	                  G_CALLBACK (extractor_get_embedded_metadata_cancel), data);
 }
@@ -2386,7 +2579,7 @@ remove_files_in_removable_media_cb (GObject      *object,
 {
 	GError *error = NULL;
 
-	tracker_miner_execute_update_finish (TRACKER_MINER (object), result, &error);
+	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (object), result, &error);
 
 	if (error) {
 		g_critical ("Could not remove files in volumes: %s", error->message);
@@ -2427,11 +2620,12 @@ miner_files_in_removable_media_remove_by_type (TrackerMinerFiles  *miner,
 		                        removable ? "true" : "false",
 		                        optical ? "true" : "false");
 
-		tracker_miner_execute_batch_update (TRACKER_MINER (miner),
-		                                    queries->str,
-		                                    NULL,
-		                                    remove_files_in_removable_media_cb,
-		                                    NULL);
+		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                        queries->str,
+                                                        G_PRIORITY_DEFAULT,
+                                                        NULL,
+                                                        remove_files_in_removable_media_cb,
+                                                        NULL);
 
 		g_string_free (queries, TRUE);
 
@@ -2468,11 +2662,12 @@ miner_files_in_removable_media_remove_by_date (TrackerMinerFiles  *miner,
 	                        "}",
 	                        date);
 
-	tracker_miner_execute_batch_update (TRACKER_MINER (miner),
-	                                    queries->str,
-	                                    NULL,
-	                                    remove_files_in_removable_media_cb,
-	                                    NULL);
+	tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
+                                                queries->str,
+                                                G_PRIORITY_DEFAULT,
+                                                NULL,
+                                                remove_files_in_removable_media_cb,
+                                                NULL);
 
 	g_string_free (queries, TRUE);
 }
