@@ -121,74 +121,86 @@ read_toc (GooList  *items,
 
 		switch (link_action->getKind()) {
 			case actionGoTo: {
-				guint title_length = item->getTitleLength ();
 				LinkGoTo *gto = dynamic_cast <LinkGoTo *> (link_action);
-				GooString *named_dest = gto->getNamedDest ();
 
-				if (title_length > 0) {
-					gchar *str = unicode_to_char (item->getTitle(),
-					                              title_length);
-					g_string_append_printf (*toc, "%s ", str);
-					g_free (str);
+				if (gto) {
+					guint title_length = item->getTitleLength ();
+					GooString *named_dest = gto->getNamedDest ();
+
+					if (title_length > 0) {
+						gchar *str = unicode_to_char (item->getTitle(),
+						                              title_length);
+						g_string_append_printf (*toc, "%s ", str);
+						g_free (str);
+					}
+
+					if (named_dest)
+						g_string_append_printf (*toc, "%s ", named_dest->getCString ());
 				}
 
-				if (named_dest)
-					g_string_append_printf (*toc, "%s ", named_dest->getCString ());
-
 				break;
 			}
 
 			case actionLaunch: {
-				guint title_length = item->getTitleLength ();
 				LinkLaunch *lan = dynamic_cast <LinkLaunch *> (link_action);
-				GooString *filen, *param;
 
-				filen = lan->getFileName();
-				param = lan->getParams();
+				if (lan) {
+					guint title_length = item->getTitleLength ();
+					GooString *filen, *param;
 
-				if (title_length > 0) {
-					gchar *str = unicode_to_char (item->getTitle(),
-					                              title_length);
-					g_string_append_printf (*toc, "%s ", str);
-					g_free (str);
+					filen = lan->getFileName();
+					param = lan->getParams();
+
+					if (title_length > 0) {
+						gchar *str = unicode_to_char (item->getTitle(),
+						                              title_length);
+						g_string_append_printf (*toc, "%s ", str);
+						g_free (str);
+					}
+
+					if (filen)
+						g_string_append_printf (*toc, "%s ", filen->getCString ());
+
+					if (param)
+						g_string_append_printf (*toc, "%s ", param->getCString ());
 				}
 
-				if (filen)
-					g_string_append_printf (*toc, "%s ", filen->getCString ());
-
-				if (param)
-					g_string_append_printf (*toc, "%s ", param->getCString ());
-
 				break;
 			}
 
 			case actionURI: {
 				LinkURI *uri = dynamic_cast <LinkURI *> (link_action);
-				GooString *muri;
 
-				muri = uri->getURI();
+				if (uri) {
+					GooString *muri;
 
-				if (muri)
-					g_string_append_printf (*toc, "%s ", muri->getCString ());
+					muri = uri->getURI();
+
+					if (muri)
+						g_string_append_printf (*toc, "%s ", muri->getCString ());
+				}
 
 				break;
 			}
 
 			case actionNamed: {
-				guint title_length = item->getTitleLength ();
 				LinkNamed *named = dynamic_cast <LinkNamed *> (link_action);
-				GooString *named_dest = named->getName ();
 
-				if (title_length > 0) {
-					gchar *str = unicode_to_char (item->getTitle(),
-					                              title_length);
-					g_string_append_printf (*toc, "%s ", str);
-					g_free (str);
+				if (named) {
+					GooString *named_dest = named->getName ();
+					guint title_length = item->getTitleLength ();
+
+					if (title_length > 0) {
+						gchar *str = unicode_to_char (item->getTitle(),
+						                              title_length);
+						g_string_append_printf (*toc, "%s ", str);
+						g_free (str);
+					}
+
+					if (named_dest)
+						g_string_append_printf (*toc, "%s ", named_dest->getCString ());
 				}
 
-				if (named_dest)
-					g_string_append_printf (*toc, "%s ", named_dest->getCString ());
-
 				break;
 			}
 
@@ -343,6 +355,7 @@ extract_content (PDFDoc *document,
 
 		delete gfx;
 		delete text_dev;
+		delete sel_text;
 	}
 
 	g_timer_destroy (timer);
@@ -487,6 +500,13 @@ extract_pdf (const gchar          *uri,
 
 	document = poppler_document_new_pdf_from_file (uri, NULL);
 
+	if (!document) {
+		g_warning ("Could not create PopplerDocument from uri:'%s', "
+		           "NULL returned without an error",
+		           uri);
+		return;
+	}
+
 	if (!document->isOk()) {
 		int fopen_errno;
 		switch (document->getErrorCode()) {
@@ -516,13 +536,6 @@ extract_pdf (const gchar          *uri,
 		return;
 	}
 
-	if (!document) {
-		g_warning ("Could not create PopplerDocument from uri:'%s', "
-		           "NULL returned without an error",
-		           uri);
-		return;
-	}
-
 	tracker_sparql_builder_predicate (metadata, "a");
 	tracker_sparql_builder_object (metadata, "nfo:PaginatedTextDocument");
 
@@ -761,6 +774,13 @@ extract_pdf (const gchar          *uri,
 		 * write what we know from Poppler.
 		 */
 		write_pdf_data (pd, metadata, keywords);
+
+		g_free (pd.keywords);
+		g_free (pd.title);
+		g_free (pd.subject);
+		g_free (pd.creation_date);
+		g_free (pd.author);
+		g_free (pd.date);
 	}
 
 	for (i = 0; i < keywords->len; i++) {
