@@ -27,7 +27,6 @@
 #include <png.h>
 
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-client/tracker-client.h>
 #include <libtracker-extract/tracker-extract.h>
 
 #define RFC1123_DATE_FORMAT "%d %B %Y %H:%M:%S %z"
@@ -240,7 +239,7 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	g_free (ed->software);
 
 	if (md.creator) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", md.creator);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.creator);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -281,7 +280,7 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	}
 
 	if (md.artist) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", md.artist);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.artist);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -355,7 +354,7 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	}
 
 	if (xd->publisher) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", xd->publisher);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", xd->publisher);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
