@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -30,10 +30,12 @@
 #include "tracker-dbus.h"
 #include "tracker-backup.h"
 #include "tracker-store.h"
+#include "tracker-resources.h"
 
 typedef struct {
 	DBusGMethodInvocation *context;
 	guint request_id;
+	gchar *journal_uri;
 } TrackerDBusMethodInfo;
 
 G_DEFINE_TYPE (TrackerBackup, tracker_backup, G_TYPE_OBJECT)
@@ -105,6 +107,41 @@ tracker_backup_save (TrackerBackup          *object,
 	g_object_unref (destination);
 }
 
+static gboolean
+backup_idle (gpointer user_data)
+{
+	TrackerDBusMethodInfo *info = user_data;
+	GFile *journal;
+	TrackerStatus *notifier;
+	TrackerBusyCallback busy_callback;
+	gpointer busy_user_data;
+
+	journal = g_file_new_for_uri (info->journal_uri);
+
+	tracker_store_set_active (FALSE);
+
+	notifier = TRACKER_STATUS (tracker_dbus_get_object (TRACKER_TYPE_STATUS));
+
+	busy_callback = tracker_status_get_callback (notifier, 
+	                                            &busy_user_data);
+
+	g_free (info->journal_uri);
+
+	tracker_data_backup_restore (journal,
+	                             backup_callback,
+	                             info, 
+	                             (GDestroyNotify) g_free,
+	                             NULL,
+	                             busy_callback,
+	                             busy_user_data);
+
+	tracker_store_set_active (TRUE);
+
+	g_object_unref (journal);
+
+	return FALSE;
+}
+
 void
 tracker_backup_restore (TrackerBackup          *object,
                         const gchar            *journal_uri,
@@ -113,7 +150,6 @@ tracker_backup_restore (TrackerBackup          *object,
 {
 	guint request_id;
 	TrackerDBusMethodInfo *info;
-	GFile *journal;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
@@ -125,13 +161,9 @@ tracker_backup_restore (TrackerBackup          *object,
 	info = g_new0 (TrackerDBusMethodInfo, 1);
 	info->request_id = request_id;
 	info->context = context;
-	journal = g_file_new_for_uri (journal_uri);
+	info->journal_uri = g_strdup (journal_uri);
 
-	tracker_data_backup_restore (journal,
-	                             backup_callback,
-	                             info, 
-	                             (GDestroyNotify) g_free);
+	g_idle_add (backup_idle, info);
 
-	g_object_unref (journal);
 }
 
